package Slic3rPrusa::XS;
use warnings;
use strict;

our $VERSION = '0.01';

# We have to load these modules in order to have Wx.pm find the correct paths
# for wxWidgets dlls on MSW.
# We avoid loading these on OS X because Wx::Load() initializes a Wx App
# automatically and it steals focus even when we're not running Slic3r in GUI mode.
# TODO: only load these when compiling with GUI support
BEGIN {
    if ($^O eq 'MSWin32') {
        eval "use Wx";
#        eval "use Wx::Html";
        eval "use Wx::Print";  # because of some Wx bug, thread creation fails if we don't have this (looks like Wx::Printout is hard-coded in some thread cleanup code)
    }
}

use Carp qw();
use XSLoader;
XSLoader::load(__PACKAGE__, $VERSION);

package Slic3rPrusa::Line;
use overload
    '@{}' => sub { $_[0]->arrayref },
    'fallback' => 1;

package Slic3rPrusa::Point;
use overload
    '@{}' => sub { $_[0]->arrayref },
    'fallback' => 1;

package Slic3rPrusa::Point3;
use overload
    '@{}' => sub { [ $_[0]->x, $_[0]->y, $_[0]->z ] },  #,
    'fallback' => 1;

sub pp {
    my ($self) = @_;
    return [ @$self ];
}

package Slic3rPrusa::Pointf;
use overload
    '@{}' => sub { $_[0]->arrayref },
    'fallback' => 1;

package Slic3rPrusa::Pointf3;
use overload
    '@{}' => sub { [ $_[0]->x, $_[0]->y, $_[0]->z ] },  #,
    'fallback' => 1;

sub pp {
    my ($self) = @_;
    return [ @$self ];
}

package Slic3rPrusa::ExPolygon;
use overload
    '@{}' => sub { $_[0]->arrayref },
    'fallback' => 1;

package Slic3rPrusa::Polyline;
use overload
    '@{}' => sub { $_[0]->arrayref },
    'fallback' => 1;

package Slic3rPrusa::Polyline::Collection;
use overload
    '@{}' => sub { $_[0]->arrayref },
    'fallback' => 1;

package Slic3rPrusa::Polygon;
use overload
    '@{}' => sub { $_[0]->arrayref },
    'fallback' => 1;

package Slic3rPrusa::ExPolygon::Collection;
use overload
    '@{}' => sub { $_[0]->arrayref },
    'fallback' => 1;

package Slic3rPrusa::ExtrusionPath::Collection;
use overload
    '@{}' => sub { $_[0]->arrayref },
    'fallback' => 1;

sub new {
    my ($class, @paths) = @_;
    
    my $self = $class->_new;
    $self->append(@paths);
    return $self;
}

package Slic3rPrusa::ExtrusionLoop;
use overload
    '@{}' => sub { $_[0]->arrayref },
    'fallback' => 1;

sub new_from_paths {
    my ($class, @paths) = @_;
    
    my $loop = $class->new;
    $loop->append($_) for @paths;
    return $loop;
}

package Slic3rPrusa::ExtrusionMultiPath;
use overload
    '@{}' => sub { $_[0]->arrayref },
    'fallback' => 1;

package Slic3rPrusa::ExtrusionPath;
use overload
    '@{}' => sub { $_[0]->arrayref },
    'fallback' => 1;

sub new {
    my ($class, %args) = @_;
    
    return $class->_new(
        $args{polyline},     # required
        $args{role},         # required
        $args{mm3_per_mm}   // die("Missing required mm3_per_mm in ExtrusionPath constructor"),
        $args{width}        // -1,
        $args{height}       // -1,
    );
}

sub clone {
    my ($self, %args) = @_;
    
    return __PACKAGE__->_new(
        $args{polyline}      // $self->polyline,
        $args{role}          // $self->role,
        $args{mm3_per_mm}    // $self->mm3_per_mm,
        $args{width}         // $self->width,
        $args{height}        // $self->height,
    );
}

package Slic3rPrusa::ExtrusionSimulator;

sub new {
    my ($class, %args) = @_;
    return $class->_new();
}

package Slic3rPrusa::Filler;

sub fill_surface {
    my ($self, $surface, %args) = @_;
    $self->set_density($args{density}) if defined($args{density});
    $self->set_dont_connect($args{dont_connect}) if defined($args{dont_connect});
    $self->set_dont_adjust($args{dont_adjust}) if defined($args{dont_adjust});
    $self->set_complete($args{complete}) if defined($args{complete});
    return $self->_fill_surface($surface);
}

package Slic3rPrusa::Flow;

sub new {
    my ($class, %args) = @_;
    
    my $self = $class->_new(
        @args{qw(width height nozzle_diameter)},
    );
    $self->set_bridge($args{bridge} // 0);
    return $self;
}

sub new_from_width {
    my ($class, %args) = @_;
    
    return $class->_new_from_width(
        @args{qw(role width nozzle_diameter layer_height bridge_flow_ratio)},
    );
}

sub new_from_spacing {
    my ($class, %args) = @_;
    
    return $class->_new_from_spacing(
        @args{qw(spacing nozzle_diameter layer_height bridge)},
    );
}

package Slic3rPrusa::Surface;

sub new {
    my ($class, %args) = @_;
    
    # defensive programming: make sure no negative bridge_angle is supplied
    die "Error: invalid negative bridge_angle\n"
        if defined $args{bridge_angle} && $args{bridge_angle} < 0;
    
    return $class->_new(
        $args{expolygon}         // (die "Missing required expolygon\n"),
        $args{surface_type}      // (die "Missing required surface_type\n"),
        $args{thickness}         // -1,
        $args{thickness_layers}  // 1,
        $args{bridge_angle}      // -1,
        $args{extra_perimeters}  // 0,
    );
}

sub clone {
    my ($self, %args) = @_;
    
    return (ref $self)->_new(
        delete $args{expolygon}         // $self->expolygon,
        delete $args{surface_type}      // $self->surface_type,
        delete $args{thickness}         // $self->thickness,
        delete $args{thickness_layers}  // $self->thickness_layers,
        delete $args{bridge_angle}      // $self->bridge_angle,
        delete $args{extra_perimeters}  // $self->extra_perimeters,
    );
}

package Slic3rPrusa::Surface::Collection;
use overload
    '@{}' => sub { $_[0]->arrayref },
    'fallback' => 1;

sub new {
    my ($class, @surfaces) = @_;
    
    my $self = $class->_new;
    $self->append($_) for @surfaces;
    return $self;
}

package Slic3rPrusa::Print::SupportMaterial2;

sub new {
    my ($class, %args) = @_;
    
    return $class->_new(
        $args{print_config},        # required
        $args{object_config},       # required
        $args{first_layer_flow},    # required
        $args{flow},                # required
        $args{interface_flow},      # required
        $args{soluble_interface}    // 0
    );
}

package Slic3rPrusa::GUI::_3DScene::GLVertexArray;
sub CLONE_SKIP { 1 }

package main;
for my $class (qw(
        Slic3rPrusa::BridgeDetector
        Slic3rPrusa::Config
        Slic3rPrusa::Config::Full
        Slic3rPrusa::Config::GCode
        Slic3rPrusa::Config::Print
        Slic3rPrusa::Config::PrintObject
        Slic3rPrusa::Config::PrintRegion
        Slic3rPrusa::Config::Static
        Slic3rPrusa::ExPolygon
        Slic3rPrusa::ExPolygon::Collection
        Slic3rPrusa::Extruder
        Slic3rPrusa::ExtrusionLoop
        Slic3rPrusa::ExtrusionMultiPath
        Slic3rPrusa::ExtrusionPath
        Slic3rPrusa::ExtrusionPath::Collection
        Slic3rPrusa::ExtrusionSimulator
        Slic3rPrusa::Filler
        Slic3rPrusa::Flow
        Slic3rPrusa::GCode
        Slic3rPrusa::GCode::AvoidCrossingPerimeters
        Slic3rPrusa::GCode::OozePrevention
        Slic3rPrusa::GCode::PlaceholderParser
        Slic3rPrusa::GCode::Wipe
        Slic3rPrusa::GCode::Writer
        Slic3rPrusa::Geometry::BoundingBox
        Slic3rPrusa::Geometry::BoundingBoxf
        Slic3rPrusa::Geometry::BoundingBoxf3
        Slic3rPrusa::Layer
        Slic3rPrusa::Layer::Region
        Slic3rPrusa::Layer::Support
        Slic3rPrusa::Line
        Slic3rPrusa::Linef3
        Slic3rPrusa::Model
        Slic3rPrusa::Model::Instance
        Slic3rPrusa::Model::Material
        Slic3rPrusa::Model::Object
        Slic3rPrusa::Model::Volume
        Slic3rPrusa::Point
        Slic3rPrusa::Point3
        Slic3rPrusa::Pointf
        Slic3rPrusa::Pointf3
        Slic3rPrusa::Polygon
        Slic3rPrusa::Polyline
        Slic3rPrusa::Polyline::Collection
        Slic3rPrusa::Print
        Slic3rPrusa::Print::Object
        Slic3rPrusa::Print::Region
        Slic3rPrusa::Print::State
        Slic3rPrusa::Surface
        Slic3rPrusa::Surface::Collection
        Slic3rPrusa::Print::SupportMaterial2
        Slic3rPrusa::TriangleMesh
    ))
{
    no strict 'refs';
    my $ref_class = $class . "::Ref";
    eval "package $ref_class; our \@ISA = '$class'; sub DESTROY {};";
}

1;
